﻿//19.03.2013
//Kamil.Hawdziejuk@uj.edu.pl


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media.Media3D;
using Common;
using Config;
using FlowsManager.Validators;
using View;

namespace FlowsManager
{
    /// <summary>
    /// Class for managing flows between component's states in home enviroment
    /// </summary>
    public class ConflictsManager
    {
        #region Fields

        View.VLogger logger;
        readonly View.VPlatform platform;
        SituationSolver solver;
        Dictionary<View.VModule, List<View.VState>> history = new Dictionary<VModule, List<View.VState>>();  

        List<IValidator> validators = new List<IValidator>();

        /// <summary>
        /// Represents the situation of the running home environment
        /// </summary>
        HomeGraph homeGraph = new HomeGraph();

        #endregion

        #region Constructor

        public ConflictsManager(View.VPlatform platform, View.VLogger logger)
        {
            this.platform = platform;
            this.logger = logger;
            this.solver = new SituationSolver(logger);
            this.validators.Add(new LimitValidator(-0.2));
            //when building adminstrator should take control of the house
            this.validators.Add(new StagnancyValidator(100));

            //when user should be warned
            this.validators.Add(new StagnancyValidator(15));
        }

        #endregion

        #region --- Flows managing ---

        /// <summary>
        /// Returns states of the running modules in the Home Environment (all modules and compoenents that are considered)
        /// </summary>
        /// <returns></returns>
        private IList<View.VState> GetSystemStates()
        {
            IList<View.VModule> runningModules = this.platform.GetModules(true);
            List<View.VState> retList = new List<VState>();

            foreach (var mod in runningModules)
            {
                retList.Add(mod.GetState());
            }

            return retList as IList<View.VState>;
        }

        private void InitializeHomeEnvironment()
        {
            //here comes adding states...
            foreach (var state in this.GetSystemStates())
            {
                this.homeGraph.AddState(state);
            }

            //here comes adding possible transitions...
        }

        #endregion

        #region --- private methods ---

        private List<View.VState> GetStatesHistory(View.VModule _vModule)
        {
            return this.history[_vModule];
        }

        private void AddToHistory(View.VModule mod)
        {
            View.VState vState = mod.GetState();
            if (!history.ContainsKey(mod))
            {
                history.Add(mod, new List<View.VState>());
            }
            View.VState vStateCopy = vState.Clone() as View.VState;
            history[mod].Add(vStateCopy);
        }
    
        /// <summary>
        /// Validates the given state
        /// </summary>
        /// <param name="vstate"></param>
        /// <returns></returns>
        private bool Validate(VState vstate)
        {
            foreach (IValidator validator in this.validators)
            {
                ProblematicSituation problem = validator.Validate(vstate);
                if (problem != null)
                {
                    if (!solver.Solve(problem, validator.Priority))
                    {
                        logger.Log("Danger state. Some other solution or solver should be fired here...");
                        return false;
                    }
                }
            }
            return true;
        }

        private void UpdateValidators(View.VModule _vModule)
        {
            foreach (IValidator validator in this.validators)
            {
                if (validator is HistoryValidator && this.history.ContainsKey(_vModule))
                {
                    (validator as HistoryValidator).History = this.history[_vModule];
                }
            }
        }

        #endregion

        public void Start()
        {
            System.Threading.Thread.Sleep(5000);
            logger.Log("Conflicts Manager has started...");
            while (true)
            {
                IList<View.VModule> list = this.platform.GetModules(true);
                if (list.Count > 0)
                {
                    foreach (var mod in list)
                    {
                        this.AddToHistory(mod);
                        this.UpdateValidators(mod);
                        View.VState vState = mod.GetState();
                        this.Validate(vState);
                    }
                }
                System.Threading.Thread.Sleep(500);
            }
        }
    }
}


