﻿//Kamil.Hawdziejuk@uj.edu.pl

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using View;
using Common;
using FlowsManager.Problems;

namespace FlowsManager.Validators
{
    /// <summary>
    /// Checks last states if they haven't changed
    /// </summary>
    class StagnancyValidator : HistoryValidator
    {
        private double eps = 0.00001;

        /// <summary>
        /// Counter of situations that haven't changed
        /// </summary>
        private int maxStagnacyCounter = 10;

        public StagnancyValidator(int _maxStagnacyCounter)
        {
            this.maxStagnacyCounter = _maxStagnacyCounter;
        }

        public override string Name
        {
            get { return "StagnancyValidator"; }
        }

        public override ProblematicSituation Validate(VState vState)
        {
            int cnt = 0;
            int n = this.History.Count;
            int max = maxStagnacyCounter;

            //we will remember here state that 'vState' is problematic with
            VState differentState = null;

            for (int i = n - 1; i >= 0; i--)
            {
                if (Math.Abs(this.History[i].ExactValue - vState.ExactValue) < eps && vState != this.History[i])
                {
                    cnt++;
                    differentState = this.History[i];
                }
                max--;
                if (max < 0)
                {
                    break;
                }
            }
            if (cnt >= maxStagnacyCounter && differentState != null)
            {
                return new StatesConflictSituation(vState, differentState);
            }
            return null;
        }

        public override SituationPriority Priority
        {
            get
            {
                if (maxStagnacyCounter < 50)
                {
                    return SituationPriority.Warning;
                }
                else
                {
                    return SituationPriority.Alarm;
                }
            }
        } 
    }
}
